# -*- coding: utf-8 -*-
import shutil
import os
from time import sleep
from config import config
import logging

logger = logging.getLogger("gettextfile.view")

from PyQt4.QtCore import QModelIndex, QString, Qt, pyqtSignal, \
    QSize, QTimer, QThread, QAbstractItemModel
from PyQt4.QtGui import QSortFilterProxyModel, QTableView, QAbstractItemView, \
    QToolBar, QIcon, QPlainTextEdit, QHBoxLayout, QFileDialog, QMessageBox, \
    QWidget, QProgressDialog

try:
    from PyQt4.QtMaemo5 import QMaemo5InformationBox
except ImportError:
    QMaemo5InformationBox = None
  
from common import CustomWindow, IconCache
from views.search_string import SearchStringDialog
from views.login import LoginDialog
from views.push_dialog import PushDialog

from gettextfile.model import GettextFile
from files.model import LocalFileModel, RemoteFileModel
from stringeditor.view import StringView
from views.about import About

class StringsetProxyModel(QSortFilterProxyModel):
    """
    Proxy model for filtering strings
    """
    def __init__(self, model):
        QSortFilterProxyModel.__init__(self)
        self.setSourceModel(model)
        self.setFilterKeyColumn(0)
        self.filterTranslated = True
        self.filterFuzzy = True
        self.filterUntranslated = True
        self.setFilterRole(-2)

    def filterAcceptsRow(self, source_row, source_parent):
        if QSortFilterProxyModel.filterAcceptsRow(self, source_row, source_parent):
            entry = self.sourceModel().objs[source_row]
            if entry.obsolete:
                return False
            elif entry.translated():
                return self.filterTranslated
            elif entry.fuzzy():
                return self.filterFuzzy
            else:
                return self.filterUntranslated
        return False

    def setFilterTranslated(self, flag):
        self.filterTranslated = flag
        self.invalidate()

    def setFilterFuzzy(self, flag):
        self.filterFuzzy = flag
        self.invalidate()

    def setFilterUntranslated(self, flag):
        self.filterUntranslated = flag
        self.invalidate()

class FilePullThread(QThread):
    setProgress = pyqtSignal(str, int)
    setSourceModel = pyqtSignal(QAbstractItemModel)
    def __init__(self, fp, parent):
        QThread.__init__(self, parent=parent)
        self.file_model = fp

    def run(self):
        target = self.parent()
        uri = self.file_model.uri()
        self.setProgress.emit(
            self.tr("Downloading..."), 1)
        self.file_model.pull()
        if not self.file_model.locked:
            self.setProgress.emit(
                self.tr("Locking..."), 2)
            self.file_model.lock()
        self.setProgress.emit(
            self.tr("Parsing..."), 3)
        source_model = GettextFile(self.file_model)
        self.setProgress.emit(
            self.tr("Processing..."), 4)
        source_model.moveToThread(self.thread())
        source_model.setParent(self)
        self.setSourceModel.emit(source_model)

        self.setProgress.emit(self.tr("Done"), 5)
        target.setWindowTitle(self.file_model.long_title)

class FilePushThread(QThread):
    setProgress = pyqtSignal(str, int)
    setSourceModel = pyqtSignal(QAbstractItemModel)
    finish = pyqtSignal()
    def __init__(self, source_model, file_model, push=True, unlock=True, write=True,
        close=False, parent=None):
        QThread.__init__(self, parent=parent)
        self.source_model = source_model
        self.file_model = file_model
        self.write = write    # Wether we need to write new file to local cache
        self.push = push      # Wether we need to push file from local cache
        self.unlock = unlock  # Wether we need to unlock pushed file

    def run(self):
        d = config.data['servers'][self.file_model.client.hostname]['profile']
        if self.write:
            self.setProgress.emit(self.tr("Generating file..."), 1)
            self.source_model.save(last_translator = "%(firstname)s %(surname)s <%(email)s>" % d)
            if self.push:
                self.setProgress.emit(
                    self.tr("Uploading file. This might take a while if VCS backend is slow..."), 2)
                self.file_model.push() # Upload file from cache to server
        if self.unlock:
            self.setProgress.emit(
                self.tr("Unlocking..."), 3)
            self.file_model.unlock()
        self.setProgress.emit(self.tr("Done..."), 4)
        self.finish.emit()

class FileView(CustomWindow):
    def editString(self, index):
        self.modified = True
        win = StringView(self.model, index,
            parent=self)
        win.show()

    def fileSave(self, write=True, push=True, unlock=True, close=True):
        if isinstance(self.file_model, RemoteFileModel):
            self.pd = QProgressDialog(parent=self)
            self.pd.setWindowTitle(self.tr("Saving file"))
            self.pd.setMaximum(5)
            # This fixes centering issues on desktop
            self.pd.setMinimumSize(QSize(500,0))
            self.pd.forceShow()
            self.pd.setWindowModality(Qt.WindowModal)
            
            thread = FilePushThread(self.source_model, self.file_model,
                write=write, push=push, unlock=unlock, parent=self)
            thread.setProgress.connect(self.setProgress, type=Qt.BlockingQueuedConnection)
            if close:
                thread.finish.connect(self.forceClose, type=Qt.QueuedConnection)
            thread.start()
        else:
            self.source_model.save()
            self.modified = False
            if QMaemo5InformationBox:
                QMaemo5InformationBox.information(self, 
                    "<font color='black'>%s saved</font>" % self.file_model.title())
            if close:
                self.forceClose()

    def fileSaveAs(self):
        filename = QFileDialog.getSaveFileName(
            self, caption="Save file as...",
            filter=QString("Gettext PO (*.po)"))

    def helpAbout(self):
        About().exec_()

    def setProgress(self, text, value):
        self.pd.setValue(value)
        self.pd.setLabelText(text)

    def resizeCells(self):
        for i in [1,2]:
            self.tableview.resizeColumnToContents(i)
        self.tableview.resizeRowsToContents()


    def setSourceModel(self, source_model):
        self.source_model = source_model
        if self.model:
            self.model.setSourceModel(source_model)
        else:
            self.model = StringsetProxyModel(source_model)

        self.tableview.setModel(self.model)
        self.resizeCells()

        # Connect signals to slots
        if isinstance(self.model, QSortFilterProxyModel):
            self.actionFilterTranslated.toggled.connect(
                self.model.setFilterTranslated)
            self.actionFilterFuzzy.toggled.connect(
                self.model.setFilterFuzzy)
            self.actionFilterUntranslated.toggled.connect(
                self.model.setFilterUntranslated)
        self.tableview.setEnabled(True)
        self.tableview.setFocus()
        

    def editFind(self, checked):
        """
        Open search string dialog and use the string in lowercase to filter 
        entries by custom datarole (-2)s
        """
        if checked:
            q = SearchStringDialog.getSearchString()
            if q:
                self.model.setFilterFixedString(q.lower())
            else:
                self.actionFilterString.setChecked(False)
        else:
            self.model.setFilterFixedString("")

    def forceClose(self):
        logger.debug("Closing window forcefully")
        self.force_close = True
        self.close()

    def closeEvent(self, e):
        logger.debug("Received close event")
        if self.force_close:
            e.accept()
            return
        r = PushDialog(remote = isinstance(self.file_model, RemoteFileModel)).exec_()
        if r == PushDialog.Cancel:
            e.ignore()
        elif r == PushDialog.Forget:
            e.accept()
        else:
            if r == PushDialog.Save:
                self.fileSave(push=False, unlock=False, close=True)
            elif r == PushDialog.Push:
                self.fileSave(push=True, unlock=False, close=True)
            elif r == PushDialog.PushAndUnlock:
                self.fileSave(push=True, unlock=True, close=True)
            elif r == PushDialog.ForgetAndUnlock:
                self.fileSave(write=False, push=False, unlock=True, close=True)
            e.ignore()          

    def __init__(self, fp=None, parent=None):
        CustomWindow.__init__(self, parent, busy=False, 
            skip_central_widget=True)
        self.setWindowTitle(self.tr("Transifex-mobile"))
        self.modified = False # Wether file is modified 
        self.force_close = False
        self.model = None # Proxied string list model
        self.source_model = None # String list model
        self.tableview = QTableView()
        self.tableview.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.tableview.setProperty("FingerScrollable", True)
        self.tableview.clicked.connect(self.editString)
        self.tableview.setSelectionMode(QAbstractItemView.NoSelection)
        self.tableview.setEnabled(False)
        self.setCentralWidget(self.tableview)
        toolbar = QToolBar(
            self.tr("Stringset"),
            self)

        # Filter translated button
        self.actionFilterTranslated = toolbar.addAction(
            IconCache.getGreenFlag(),
            self.tr("Translated"))
        self.actionFilterTranslated.setCheckable(True)
        self.actionFilterTranslated.setChecked(True)

        # Filter fuzzy button
        self.actionFilterFuzzy = toolbar.addAction(
            IconCache.getBlueFlag(),
            self.tr("Fuzzy"))
        self.actionFilterFuzzy.setCheckable(True)
        self.actionFilterFuzzy.setChecked(True)

        # Filter untranslated button
        self.actionFilterUntranslated = toolbar.addAction(
            IconCache.getRedFlag(),
            self.tr("Untranslated"))
        self.actionFilterUntranslated.setCheckable(True)
        self.actionFilterUntranslated.setChecked(True)

        # Search string button
        self.actionFilterString = toolbar.addAction(
            IconCache.getSearch(),
            self.tr("Search string"))
        self.addToolBar(toolbar)
        self.actionFilterString.setCheckable(True)
        self.actionFilterString.triggered.connect(self.editFind)

        # Allow some actions only once in "local" editor mode
#        if not fp or isinstance(fp, LocalFileModel):
#            menuFile = self.menuBar().addMenu(self.tr("&File"))
#            menuFile.addAction(self.tr("&Open..."), self.fileOpen)
#            menuFile.addAction(self.tr("&Save"), self.fileSave)
#            menuFile.addAction(self.tr("Save &as..."), self.fileSaveAs)

        # Edit menu
        menuEdit = self.menuBar().addMenu(self.tr("&Edit"))
        menuEdit.addAction(self.actionFilterString)

        # Help menu
        menuHelp = self.menuBar().addMenu(self.tr("&Help"))
        menuHelp.addAction(QIcon(), self.tr("&About"), self.helpAbout)

        # If we have file, load it
        if fp:
            self.pd = QProgressDialog(parent=self)
            self.pd.setWindowTitle(self.tr("Loading file"))
            self.pd.setMaximum(5)
            # This fixes centering issues on desktop
            self.pd.setMinimumSize(QSize(500,0))
            self.pd.forceShow()
            self.pd.setWindowModality(Qt.WindowModal)
            self.pd.setValue(0)

            thread = FilePullThread(fp, parent=self)
            self.file_model = fp
            thread.setProgress.connect(self.setProgress, type=Qt.BlockingQueuedConnection)
            thread.setSourceModel.connect(self.setSourceModel, type=Qt.BlockingQueuedConnection)
            thread.start()
